#! /usr/bin/env python3

# ######################################################################## #
# File:     tests/test_time.py
#
# Purpose:  Unit-test for `asynkio.time.Duration`.
#
# Created:  25th July 2025
# Updated:  24th August 2025
#
# Copyright (c) Matthew Wilson, Synesis Information Systems Pty Ltd
# All rights reserved
#
# ######################################################################## #


import math
from asynkio.time import (
    Duration,
    Instant,
)


def test_Duration_DURATION_0():

    duration = Duration(0, 0)

    assert 0 == duration.as_nanos()
    assert 0 == duration.as_micros()
    assert 0 == duration.as_millis()
    assert 0 == duration.as_secs()
    assert 0.0 == duration.as_secs_f()

    assert 0 == duration.subsec_nanos()
    assert 0 == duration.subsec_micros()
    assert 0 == duration.subsec_millis()

    assert "0s" == str(duration)


def test_Duration_DURATION_1_SECOND():

    duration = Duration.from_secs(1)

    assert 1_000_000_000 == duration.as_nanos()
    assert 1_000_000 == duration.as_micros()
    assert 1_000 == duration.as_millis()
    assert 1 == duration.as_secs()
    assert math.isclose(1.0, duration.as_secs_f())

    assert 0 == duration.subsec_nanos()
    assert 0 == duration.subsec_micros()
    assert 0 == duration.subsec_millis()

    assert "1s" == str(duration)


def test_Duration_DURATION_123_MILLISECONDS():

    duration = Duration.from_millis(123)

    assert 123_000_000 == duration.as_nanos()
    assert 123_000 == duration.as_micros()
    assert 123 == duration.as_millis()
    assert 0 == duration.as_secs()
    assert math.isclose(0.123, duration.as_secs_f())

    assert 123_000_000 == duration.subsec_nanos()
    assert 123_000 == duration.subsec_micros()
    assert 123 == duration.subsec_millis()

    assert "123ms" == str(duration)


def test_Duration_DURATION_123_456_789_NANOSECONDS():

    duration = Duration.from_nanos(123_456_789)

    assert 123_456_789 == duration.as_nanos()
    assert 123_456 == duration.as_micros()
    assert 123 == duration.as_millis()
    assert 0 == duration.as_secs()
    assert math.isclose(0.123_456_789, duration.as_secs_f())

    assert 123_456_789 == duration.subsec_nanos()
    assert 123_456 == duration.subsec_micros()
    assert 123 == duration.subsec_millis()

    assert "123.5ms" == str(duration)


def test_Duration_STRINGS():

    assert "9ns" == str(Duration.from_nanos(9))
    assert "89ns" == str(Duration.from_nanos(89))
    assert "789ns" == str(Duration.from_nanos(789))

    assert "6.789µs" == str(Duration.from_nanos(6_789))
    assert "56.79µs" == str(Duration.from_nanos(56_789))
    assert "456.8µs" == str(Duration.from_nanos(456_789))

    assert "3.457ms" == str(Duration.from_nanos(3_456_789))
    assert "23.46ms" == str(Duration.from_nanos(23_456_789))
    assert "123.5ms" == str(Duration.from_nanos(123_456_789))

    assert "9.123s" == str(Duration.from_nanos(9_123_456_789))


    assert "9ns" == str(Duration.from_nanos(9))
    assert "80ns" == str(Duration.from_nanos(80))
    assert "700ns" == str(Duration.from_nanos(700))

    assert "6µs" == str(Duration.from_nanos(6_000))
    assert "50µs" == str(Duration.from_nanos(50_000))
    assert "400µs" == str(Duration.from_nanos(400_000))

    assert "3ms" == str(Duration.from_nanos(3_000_000))
    assert "20ms" == str(Duration.from_nanos(20_000_000))
    assert "100ms" == str(Duration.from_nanos(100_000_000))

    assert "9s" == str(Duration.from_nanos(9_000_000_000))
    assert "10s" == str(Duration.from_nanos(10_000_000_000))
    assert "200s" == str(Duration.from_nanos(200_000_000_000))
    assert "3000s" == str(Duration.from_nanos(3_000_000_000_000))
    assert "40000s" == str(Duration.from_nanos(40_000_000_000_000))


def test_Instant_CREATE():

    instant_1 = Instant.now()
    instant_2 = Instant.now()

    assert instant_1 < instant_2
    assert instant_1 <= instant_2

    assert instant_2 > instant_1
    assert instant_2 >= instant_1

    duration_1_2 = instant_2 - instant_1

    max_duration_delta = 10

    assert duration_1_2.as_micros() < max_duration_delta, f"{duration_1_2.as_micros()} should be < {max_duration_delta}"


    instant_3 = instant_1 + Duration.from_micros(123)

    duration_1_3 = instant_3 - instant_1

    assert 123_000 == duration_1_3.as_nanos()



#! /usr/bin/env python3

# ######################################################################## #
# File:     tests/test_timer.py
#
# Created:  25th July 2025
# Updated:  25th July 2025
#
# Author:   Matthew Wilson
#
# Copyright (c) Matthew Wilson, Synesis Information Systems Pty Ltd
# All rights reserved
#
# ######################################################################## #


import asyncio
import pyclasp as clasp
import diagnosticism as d
import diagnosticism.severity as sev
import time


async def run_sleep():

    d.trace()

    t1 = time.time_ns()

    while True:

        await asyncio.sleep(1)

        t2 = time.time_ns()

        d.log(sev.NOTICE, f"{d.func()} slept for {t2 - t1}")

        t1 = t2


async def run_delay():

    d.trace()

    t0 = time.perf_counter_ns()

    count = 0

    t1 = time.time_ns()

    delta = 1

    while True:

        await asyncio.sleep(delta)

        t2 = time.time_ns()

        d.log(sev.NOTICE, f"{d.func()} slept for {t2 - t1}")

        t1 = t2

        delta = 0.8


async def main():

    d.trace()

    await asyncio.gather(run_sleep(), run_delay())


if __name__ == "__main__":

    d.enable_logging('ENABLE_LOGGING', True)
    d.enable_tracing('ENABLE_TRACING', True)

    asyncio.run(main())



import time
from typing import Self

class Duration:
    """
    Represents a span of time.
    """

    def __init__(
        self,
        from_ns,
        to_ns,
    ):
        """
        Creates a new instance from the number of nanoseconds represented by
        `to_ns - from_ns`.
        """

        assert from_ns <= to_ns

        self._duration = to_ns - from_ns

    @staticmethod
    def from_nanos(t_ns : int) -> Self:
        """
        Creates a new instance from the specified number of nanoseconds.
        """

        return Duration(0, t_ns)

    @staticmethod
    def from_micros(t_us : int) -> Self:
        """
        Creates a new instance from the specified number of microseconds.
        """

        return Duration(0, t_us * 1_000)

    @staticmethod
    def from_millis(t_ms : int) -> Self:
        """
        Creates a new instance from the specified number of milliseconds.
        """

        return Duration(0, t_ms * 1_000_000)

    @staticmethod
    def from_secs(t_s : int) -> Self:
        """
        Creates a new instance from the specified number of seconds.
        """

        return Duration(0, t_s * 1_000_000_000)

    def as_nanos(self) -> int:
        """
        Total number of whole nanoseconds contained by this instance.
        """

        return self._duration

    def as_micros(self) -> int:
        """
        Total number of whole microseconds contained by this instance.
        """

        return self._duration // 1_000

    def as_millis(self) -> int:
        """
        Total number of whole milliseconds contained by this instance.
        """

        return self._duration // 1_000_000

    def as_secs(self) -> int:
        """
        Total number of whole seconds contained by this instance.
        """

        return self._duration // 1_000_000_000

    def as_secs_f(self) -> float:
        """
        The fractional number of seconds contained by this instance.
        """

        return self._duration / 1_000_000_000

    def subsec_nanos(self) -> int:
        """
        The fractional part of this instance, in nanoseconds.
        """

        return self._duration % 1_000_000_000

    def subsec_micros(self) -> int:
        """
        The fractional part of this instance, in microseconds.
        """

        return (self._duration % 1_000_000_000) // 1_000

    def subsec_millis(self) -> int:
        """
        The fractional part of this instance, in milliseconds.
        """

        return (self._duration % 1_000_000_000) // 1_000_000

    def __format__(self, format_spec) -> str:

        v = self._duration

        if v == 0:

            return "0s"

        if v >= 1_000_000:

            if v >= 1_000_000_000:

                q, r = divmod(v, 1_000_000_000)

                if r > 1_000_000:

                    return f"{self._duration / 1_000_000_000.0:.4}s"
                else:

                    return f"{self._duration // 1_000_000_000}s"
            else:

                q, r = divmod(v, 1_000_000)

                if r > 1_000:

                    return f"{self._duration / 1_000_000.0:.4}ms"
                else:

                    return f"{self._duration // 1_000_000}ms"

        if v >= 1_000:

            q, r = divmod(v, 1_000)

            if r > 1:

                return f"{self._duration / 1_000.0:.4}µs"
            else:

                return f"{self._duration // 1_000}µs"

        return f"{self._duration}ns"

    def __str__(self):

        return self.__format__('')


class Instant:
    """
    Represents a moment in time.
    """

    def __init__(self, t_ns):
        """
        Initialises with the given time instant (specified in nanoseconds).
        """

        self._t = t_ns

    @staticmethod
    def now() -> Self:
        """
        Initialises with the current time instant.
        """

        t_now_ns = time.time_ns()

        return Instant(t_now_ns)

    def __lt__(self, other):
        """
        Determines whether the called instance is less-than the `other`
        instance of `Instant`.
        """

        if isinstance(other, Instant):

            return self._t < other._t

        raise NotImplemented

    def __le__(self, other):
        """
        Determines whether the called instance is less-than-or-equal-to the
        `other` instance of `Instant`.
        """

        if isinstance(other, Instant):

            return self._t <= other._t

        raise NotImplemented

    def __sub__(self, other):
        """
        Subtracts the `other` - which may be an instance of `Instant` or an
        instance of `Duration` - from the called instance.
        """

        if isinstance(other, Instant):

            return Duration(other._t, self._t)

        if isinstance(other, Duration):

            return Instant(self._t - other.as_nanos())

        return NotImplemented

    def __add__(self, other):
        """
        Adds the `other` instance of `Duration` to the called instance.
        """

        if isinstance(other, Duration):

            return Instant(self._t + other.as_nanos())

        return NotImplemented

